﻿/* erikhallander@gmail.com */

package com {
	
	import flash.display.*;
	import flash.utils.setTimeout;
	import flash.utils.ByteArray;
	import flash.events.*;
	import flash.geom.Rectangle;
	import flash.geom.Point;
	import flash.geom.Matrix;
	import flash.display.DisplayObject;
	import fl.transitions.easing.None;
	import fl.transitions.easing.Regular;
	
	import com.greensock.TweenLite;
	
	import flash.system.System;
	
	final public class PixelExploder extends Sprite
	{
		private var orgClip:DisplayObject;
		private var orgBMD:BitmapData;
		
		private var baseColor:uint;
		private var pixelSize:Number;
		
		private var pixelStore:Vector.<PixelParticle> = new Vector.<PixelParticle>;
		
		private var mHeight:Number;
		private var mWidth:Number;
		
		/* Const */
		public function PixelExploder(_orgClip:DisplayObject, aBaseColor:uint = 0x000000, aPixelSize:Number = 2, aWidth:Number = 0, aHeight:Number = 0)
		{
			baseColor = aBaseColor;
			pixelSize = aPixelSize;
			orgClip = _orgClip;
			
			mHeight = aHeight;
			mWidth = aWidth;
			
			x = orgClip.x;
			y = orgClip.y;
			mouseChildren = false;
			mouseEnabled = false;
			
			setupPixels();
		}
		
		public function explode(time:Number = .7):void
		{
			orgClip.visible = false;
			visible = true;
			alpha = 1;
			var cPixel:PixelParticle;
			if (0 < time) {
				for each(cPixel in pixelStore) {
					TweenLite.to(cPixel,time,{x:cPixel.tox,y:cPixel.toy,ease:None.easeOut});
				}
				TweenLite.to(this,time,{alpha: 0,ease:None.easeOut, onComplete: finExplode});
			}
			else {
				for each(cPixel in pixelStore) {
					cPixel.x = cPixel.tox;
					cPixel.y = cPixel.toy;
				}
				alpha = 0;
				finExplode();
			}
		}
		private function finExplode():void {
			dispatchEvent(new Event("EXPLODE_DONE"));
		};
		
		public function retract(time:Number = 1):void
		{
			var cPixel:PixelParticle;
			if (0 < time) {
				for each(cPixel in pixelStore) {
					TweenLite.to(cPixel,time,{y:cPixel.origY,x:cPixel.origX,ease:Regular.easeIn});
				}
				alpha = 0;
				TweenLite.to(this,time,{alpha: 1,ease:Regular.easeIn, onComplete: finRetract});
			}
			else {
				finRetract();
			}
		}
		private function finRetract():void
		{
			orgClip.visible = true;
			visible = false;
			dispatchEvent(new Event("RETRACT_DONE"));
		}
		
		private function setupPixels():void
		{
			var h:int = (mHeight > 0) ? mHeight : orgClip.height;
			var w:int = (mWidth > 0) ? mWidth : orgClip.width;
			
			orgBMD = new BitmapData(w, h, false);
			orgBMD.draw(orgClip);
			
			// :HACK: removes odd black line from bottom of bitmap.
			--h;
			--w;
			var cPixel:PixelParticle;
			for(var i:int = 0; i<h; i+=pixelSize)
			{
				for(var j:int = 0; j<w; j+=pixelSize)
				{
					var angle:Number = Math.random()*360;
					var radius:Number = Math.random()*(h*2);
					
					if (orgBMD.getPixel(j,i) === baseColor)
						continue;
					
					cPixel = new PixelParticle(new BitmapData(pixelSize, pixelSize, false, orgBMD.getPixel(j,i)), j, i);
					cPixel.x = j;
					cPixel.y = i;
					cPixel.tox = Math.round(j+Math.cos(angle)*radius);
					cPixel.toy = Math.round(i+Math.sin(angle)*radius);
					
					addChild(cPixel);
					pixelStore.push(cPixel);
					
				}
			}
			
			visible = false;
			pixelStore.fixed = true;
		}
	}
}